using System.Collections;
using LibNoise.Unity;
using LibNoise.Unity.Generator;
using LibNoise.Unity.Operator;
using System.IO;
using System;
using UnityEngine;

public class ApplyCloud : Affector {
        private Noise2D m_noiseMap = null;
        private Texture2D[] m_textures = new Texture2D[3];
        public int resolution = 64; 
        public NoiseType noise = NoiseType.RidgedMultifractal;
		float[] clipping;
		float[] new_clipping;
		string[] clipping_labels;
	
	ModuleBase moduleBase;
	
	Color randColor1;
	Color randColor2;
	Color randColor3;
	
	bool enableGui = false;
	
	bool genGrad = false;
	LibNoise.Unity.Gradient coloredGrad;
	LibNoise.Unity.Gradient monocromeGrad;
	
    
    override public void Start() {
		base.Start();
		clipping = new float[] {0f, 0f, 1f};
		new_clipping = new float[] {0f, 0f, 10f};
		clipping_labels = new string[] {"horizontal offset", "vertical offset", "zoom"};
		
    	Generate();
    }
	
	override public void Update() {
		base.Update();
		
		new_clipping[0] += UnityEngine.Random.Range(1, 20)/ 10000f;
		//new_clipping[1] += .02f;
		for(int i=0; i < 3; i++) {
			if (clipping[i] != new_clipping[i]) {
				clipping[i] = new_clipping[i];
				Generate();
				break;
			}
		}
	}	
    
    public void OnGUI() {
		if (!this.enableGui)
			return;
		
    	int y = 0;
    	foreach ( string i in System.Enum.GetNames(typeof(NoiseType)) ) {
    		if (GUI.Button(new Rect(0,y,125,20), i) ) {
    			noise = (NoiseType) Enum.Parse(typeof(NoiseType), i);
				this.moduleBase = null;
    			Generate();
    		}
    		y+=20;
    	}
		
		for(int i=0; i < 3; i++) {
			GUI.Label(new Rect(5,i*20 + 120,100,20), clipping_labels[i] + ": " + new_clipping[i].ToString("F2"));
		}
		
		new_clipping[0] = GUI.HorizontalSlider (new Rect(100, 120, 500,20), new_clipping[0], 0.0f, 100f);
		new_clipping[1] = GUI.HorizontalSlider (new Rect(100, 140, 500,20), new_clipping[1], 0.0f, 100);
		new_clipping[2] = GUI.HorizontalSlider (new Rect(100, 160, 500,20), new_clipping[2], 0.01f,100f);

    }
    	
    public void Generate() 
	{	
		if (!this.renderer.enabled)
			return;
		
        // Create the module network
		if (this.moduleBase == null)
		{
		    switch(noise) {
	            case NoiseType.Billow:	
	            	moduleBase = new Billow();
	            	break;
            	
	            case NoiseType.RidgedMultifractal:	
	            	moduleBase = new RidgedMultifractal(1.0, 2.0, 6,  UnityEngine.Random.Range(1, 4000000), QualityMode.High);
	            	break;   
            	
	            case NoiseType.Voronoi:	
	            	moduleBase = new Voronoi();
	            	break;             	         	
            	
              	case NoiseType.Mix:            	
	            	Perlin perlin = new Perlin();
	            	RidgedMultifractal rigged = new RidgedMultifractal();
	            	moduleBase = new Add(perlin, rigged);
	            	break;
            	
            	default:
	            	moduleBase = new Perlin(1.0f, 2.0f, 0.5f, 6, UnityEngine.Random.Range(1, 4000000), QualityMode.High);
	            	break;            	
            }
			
		
			this.randColor1 = new Color(UnityEngine.Random.value, UnityEngine.Random.value, UnityEngine.Random.value, 0f);
			this.randColor2 = randColor1;
			this.randColor2.a = .7f;
			
			this.randColor3 = new Color(UnityEngine.Random.value, UnityEngine.Random.value, UnityEngine.Random.value, 0.8f);
			
		}

        // Initialize the noise map
        this.m_noiseMap = new Noise2D(resolution, resolution, this.moduleBase);
        this.m_noiseMap.GeneratePlanar(
            clipping[0] - .5/clipping[2], 
            clipping[0] + .5/clipping[2], 
			clipping[1] - .5/clipping[2], 
            clipping[1] + .5/clipping[2]
		);
		
		
		if (!this.genGrad)
		{
			this.coloredGrad = new LibNoise.Unity.Gradient(Color.white);
			this.coloredGrad[-1.0] = new Color(1f, 1f, 1f, 0f);
			this.coloredGrad[0.0] = randColor1;
			this.coloredGrad[0.7] = randColor2;
			this.coloredGrad[1.0] = randColor3;
			
			this.monocromeGrad = new LibNoise.Unity.Gradient(Color.white);
			this.monocromeGrad[-1.0] = new Color(1f, 1f, 1f, 0f);
			this.monocromeGrad[0.0] = new Color(.2f, .2f, .2f, 0f);
			this.monocromeGrad[0.7] = new Color(.4f, .4f, .4f, 0.7f);
			this.monocromeGrad[1.0] = new Color(1f, 1f, 1f, 0.8f);
		}
		
		      // Generate the textures
        this.m_textures[0] = this.m_noiseMap.GetTexture(this.complexity > 2 ? this.coloredGrad : this.monocromeGrad);
		m_noiseMap.Border = 0;
        this.m_textures[0].Apply();
        
        //this.m_textures[1] = this.m_noiseMap.GetTexture(LibNoise.Unity.Gradient.Terrain);
        //this.m_textures[1].Apply();
         
        //this.m_textures[2] = this.m_noiseMap.GetNormalMap(3.0f);
		// this.m_textures[2].Apply();
		 
		 //display on plane
		renderer.material.mainTexture = m_textures[0];
		//renderer.material.SetTexture("_BumpMap", m_textures[2]);
    }    
	
	
	override public bool Affect(int power, RaycastHit hit)
	{
		// Negs
		if (power <= -4 && this.complexity != 1)
		{
			this.complexity--;
		}
		else if (power <= -3 && this.complexity != 2)
		{
			this.complexity--;
		}
		// Pos
		else if (power > 0 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity++;
		}
		else
		{
			return false;	
		}
		
		switch (this.complexity)
		{
			case 1:	
				this.renderer.enabled = false;		
				this.collider.enabled = false;
				
				break;
			case 2:	
						        
				break;
		}
		return true;
	}
}