using System.Collections;
using LibNoise.Unity;
using LibNoise.Unity.Generator;
using LibNoise.Unity.Operator;
using System.IO;
using System;
using UnityEngine;
using System.Collections.Generic;

//public enum NoiseType {Perlin, Billow, RidgedMultifractal, Voronoi, Mix};

public class ApplyTerrain : Affector {
    private Noise2D m_noiseMap = null;
    private Texture2D[] m_textures = new Texture2D[3];
    public int resolution = 64; 
    public NoiseType noise = NoiseType.Perlin;
	float[] clipping;
	float[] new_clipping;
	string[] clipping_labels;
	
	bool enableGui = false;
	
	bool allWhite = false;
	
	bool whipedHeights = false;
    
    override public void Start() {
		base.Start();
		clipping = new float[] {0f, 0f, 1f};
		new_clipping = new float[] {0f, 0f, 1f};
		clipping_labels = new string[] {"horizontal offset", "vertical offset", "zoom"};
		
    	Generate();
    }
	
	override public void Update() {
		if (this.enableGui)
		{
			for(int i=0; i < 3; i++) {
				if (clipping[i] != new_clipping[i]) {
					clipping[i] = new_clipping[i];
					Generate();
					break;
				}
			}
		}
	}
	
	override public bool Affect(int power, RaycastHit hit)
	{
		var terrain = this.GetComponent<Terrain>();
		
		if (power == -1 && hit.distance < 50f)
		{
			var changed = false;
			var tt = new List<TreeInstance>(terrain.terrainData.treeInstances);
			for (var t = 0; t < tt.Count; t++)
			{
				var bla = tt[t];
				var ttDistance =  Vector3.Distance(Vector3.Scale(bla.position, terrain.terrainData.size) + terrain.transform.position, hit.point);
				if (ttDistance < 10)
				{
					// switch to black
					if (bla.prototypeIndex == 0)
					{
						if (bla.heightScale != 2 && bla.widthScale != 2)
							changed = true;
						bla.heightScale = 2;
						bla.widthScale = 2;
					}
					else if (bla.prototypeIndex == 1)
					{
						if (bla.heightScale != 7 && bla.widthScale != 2)
							changed = true;
						bla.heightScale = 7;
						bla.widthScale = 2;
					}
				}
				tt[t] = bla;
			}
			if (changed)
				terrain.terrainData.treeInstances = tt.ToArray();
			return changed;
		}
		else if (power == -4 && this.allWhite)
		{
			this.whipedHeights = true;
			
			this.gameObject.SetActive(false);
			
			this.playerPower.annihilatedThings++;
			this.playerPower.nukedTerrain = true;
		}		
		else if (power == -2 && hit.distance < 100f || power <= -3)
		{
			Texture2D tmpTexture = terrain.terrainData.splatPrototypes[0].texture;
			
			var pixelOffset = 1;
			var range = 60;
			if (power == -3)
			{
				pixelOffset = 2;
				range = 120;
			}
			else if (power == -4)
			{
				pixelOffset = 8;
				range = 240;
			}
			
			var change = false;
			for (var i = -pixelOffset; i <= pixelOffset; i++)
			{
				for (var j = -pixelOffset; j <= pixelOffset; j++)
				{
					if (Mathf.Abs(j) + Mathf.Abs(i) > pixelOffset)
						continue;
					
					change = change || tmpTexture.GetPixel(Mathf.FloorToInt(hit.textureCoord.x * 64f)+i, Mathf.FloorToInt(hit.textureCoord.y * 64f)+j) != Color.white;
					
					tmpTexture.SetPixel(Mathf.FloorToInt(hit.textureCoord.x * 64f)+i, Mathf.FloorToInt(hit.textureCoord.y * 64f)+j, Color.white);	
				}
			}

			if (!change && power > -4)
				return false;
			
			if (power >= -4 && !this.allWhite)
			{
				this.allWhite = true;
				var whiteCount = 0;
				for (var i = 0; i < tmpTexture.height; i++)
				{
					for (var j = 0; j < tmpTexture.width; j++)
					{
						if (tmpTexture.GetPixel(i, j) != Color.white)
							this.allWhite = false;
						else
							whiteCount++;
					}
				}
				if (this.allWhite || whiteCount > tmpTexture.height*tmpTexture.width*.95f)
				{
					this.playerPower.annihilatedThings++;
					this.allWhite = true;
				}
			}
			
			tmpTexture.Apply(true);
			
			
			var treeChange = false;
			var tt = new List<TreeInstance>();
			foreach (var t in terrain.terrainData.treeInstances)
			{
				var bla = t;
				var ttDistance =  Vector3.Distance(Vector3.Scale(bla.position, terrain.terrainData.size) + terrain.transform.position, hit.point);
				if (ttDistance < range)
				{
					// Large power = trip trees
					if (power <= -4)
					{
						treeChange = true;
						continue;
					}
					
					// switch to black
					if (bla.prototypeIndex == 0)
					{
						bla.heightScale = 2;
						bla.widthScale = 2;
						bla.prototypeIndex = 2;
						treeChange = true;
					}
					else if (bla.prototypeIndex == 1)
					{
						bla.heightScale = 7;
						bla.widthScale = 2;
						bla.prototypeIndex = 3;
						treeChange = true;
					}
					
				}
				tt.Add(bla);
			}
			
			if (tt.Count == 0 && terrain.terrainData.treeInstances.Length != 0)
				this.playerPower.annihilatedThings++;
			
			if (treeChange)
				terrain.terrainData.treeInstances = tt.ToArray();
			
			
			
			//terrain.terrainData.RefreshPrototypes();
			
			//terrain.Flush();
			
			/*
			SplatPrototype[] tmpSplats = new SplatPrototype[1];
			tmpSplats[0] = new SplatPrototype();
	        //tmpSplats[0].texture = new Texture2D(1,1);
			//tmpSplats[0].texture.SetPixel(0, 0, new Color(UnityEngine.Random.value, UnityEngine.Random.value, UnityEngine.Random.value));
			//tmpSplats[0].tileSize = new Vector2(1, 1);
			tmpSplats[0].texture = tmpTexture;
			tmpSplats[0].texture.filterMode = FilterMode.Trilinear;
			tmpSplats[0].tileSize = new Vector2(terrain.terrainData.heightmapWidth*4, terrain.terrainData.heightmapHeight*4);
			tmpSplats[0].tileOffset = new Vector2(0, 0); 
			
			tmpSplats[0].texture.Apply(true);
			terrain.terrainData.splatPrototypes = tmpSplats;
			
			terrain.Flush();
			//hit
			//this.GetComponent<Terrain>().terrainData;
			*/
			return (change || treeChange) && power > -3 && UnityEngine.Random.value < 0.25f;
		}
		else if (power == -2 && hit.distance < 50f)
		{
			
		}
		else
		{
			return false;	
		}
		
		return true;		
	}	
    
    public void OnGUI() {
		if (!this.enableGui)
			return;

		int y = 0;
    	foreach ( string i in System.Enum.GetNames(typeof(NoiseType)) ) {
    		if (GUI.Button(new Rect(0,y,125,20), i) ) {
    			noise = (NoiseType) Enum.Parse(typeof(NoiseType), i);
    			Generate();
    		}
    		y+=20;
    	}
		
		for(int i=0; i < 3; i++) {
			GUI.Label(new Rect(5,i*20 + 120,100,20), clipping_labels[i] + ": " + new_clipping[i].ToString("F2"));
		}
		
		new_clipping[0] = GUI.HorizontalSlider (new Rect(100, 120, 500,20), new_clipping[0], 0.0f, 100f);
		new_clipping[1] = GUI.HorizontalSlider (new Rect(100, 140, 500,20), new_clipping[1], 0.0f, 100);
		new_clipping[2] = GUI.HorizontalSlider (new Rect(100, 160, 500,20), new_clipping[2], 0.1f, 5.0f);
		
    }
    	
    public void Generate() {	
        // Create the module network
        ModuleBase moduleBase;
        switch(noise) {
            case NoiseType.Billow:	
            	moduleBase = new Billow();
            	break;
        	
            case NoiseType.RidgedMultifractal:	
            	moduleBase = new RidgedMultifractal(1.0, 2.0, 6,  UnityEngine.Random.Range(1, 4000000), QualityMode.High);
            	break;   
        	
            case NoiseType.Voronoi:	
            	moduleBase = new Voronoi();
            	break;             	         	
        	
          	case NoiseType.Mix:            	
            	Perlin perlin = new Perlin();
            	RidgedMultifractal rigged = new RidgedMultifractal(1.0, 2.0, 6,  UnityEngine.Random.Range(1, 4000000), QualityMode.High);
	            	
            	moduleBase = new Add(perlin, rigged);
            	break;
        	
        	default:
            	moduleBase = new Perlin();
            	break;            	
        }

        // Initialize the noise map
        this.m_noiseMap = new Noise2D(resolution, resolution, moduleBase);
		
        // Generate the textures
		/*
        this.m_textures[0] = this.m_noiseMap.GetTexture(LibNoise.Unity.Gradient.Grayscale);
        this.m_textures[0].Apply();
        
        this.m_textures[1] = this.m_noiseMap.GetTexture(LibNoise.Unity.Gradient.Terrain);
        this.m_textures[1].Apply();
         
        this.m_textures[2] = this.m_noiseMap.GetNormalMap(3.0f);
		 this.m_textures[2].Apply();
		 
		 //display on plane
		renderer.material.mainTexture = m_textures[0];
		renderer.material.SetTexture("_BumpMap", m_textures[2]);
		*/
		//var heights = new float[this.m_noiseMap.Width,this.m_noiseMap.Height];
		
		var terrain = this.GetComponent<Terrain>();
		
		var heights = terrain.terrainData.GetHeights(0, 0, terrain.terrainData.heightmapWidth, terrain.terrainData.heightmapHeight);
		
		for (var x = 0; x < terrain.terrainData.heightmapWidth; x++)
		{
			for (var y = 0; y < terrain.terrainData.heightmapHeight; y++)
			{
				var val = (float)this.m_noiseMap.Generator.GetValue(new Vector3(x/100f, 0, y/100f));
				heights[x,y] = Mathf.Clamp((val+1.25f)/2.5f, 0, 1.5f)/10;
			}
		}
		terrain.terrainData.SetHeights(0, 0, heights);
		
		
		this.m_textures[0] = this.m_noiseMap.GetTexture(LibNoise.Unity.Gradient.Grayscale);
        this.m_textures[0].Apply();
        
		
		// Random gradient
		
		var grad = new LibNoise.Unity.Gradient(Color.white);
		var rc = new Color(UnityEngine.Random.value, UnityEngine.Random.value, UnityEngine.Random.value);
		grad[-1.0] = rc * 0.2f;
		grad[0.0] = rc * .3f;
		grad[0.2] = rc * .2f;
		grad[1.0] = rc * 1f;

		
		
		SplatPrototype[] tmpSplats = new SplatPrototype[1];
		tmpSplats[0] = new SplatPrototype();
        //tmpSplats[0].texture = new Texture2D(1,1);
		//tmpSplats[0].texture.SetPixel(0, 0, new Color(UnityEngine.Random.value, UnityEngine.Random.value, UnityEngine.Random.value));
		//tmpSplats[0].tileSize = new Vector2(1, 1);
		
		// Texture
		this.m_noiseMap.GeneratePlanar(0, (terrain.terrainData.heightmapWidth+10)/100f, 0, (terrain.terrainData.heightmapHeight+10)/100f);
		tmpSplats[0].texture = this.m_noiseMap.GetTexture(grad, true);
		tmpSplats[0].texture.filterMode = FilterMode.Trilinear;
		tmpSplats[0].tileSize = new Vector2(terrain.terrainData.heightmapWidth*4, terrain.terrainData.heightmapHeight*4);
		tmpSplats[0].tileOffset = new Vector2(0, 0); 
		
		tmpSplats[0].texture.Apply(true);
		terrain.terrainData.splatPrototypes = tmpSplats;

		
		var tt = new List<TreeInstance>(terrain.terrainData.treeInstances);
		for (var t = 0; t < tt.Count; t++)
		{
			var bla = tt[t];
			bla.heightScale = UnityEngine.Random.Range(5, UnityEngine.Random.value < .1f ? 27 : 12);
			bla.widthScale = 1 + UnityEngine.Random.value * 2;
			bla.prototypeIndex = UnityEngine.Random.Range(0, 2);
			tt[t] = bla;
		}
		terrain.terrainData.treeInstances = tt.ToArray();
		
		
        
		terrain.terrainData.RefreshPrototypes();
		terrain.Flush();
    }
    
}