using UnityEngine;
using System.Collections;

public class BigCubeBehavior : Affector {
	
	public Vector3 targetLocation;
	public Transform target;
	public float forceSize = 100f;
	public float maxSpeed = 10f;
	
	public float upForce = 90f;
	public float spikeUpForce = 120f;
	
	public float followForceSize = 30f;
	public float followMaxSpeed = 30f;
	
	public float size = 50f;
	
	// Use this for initialization
	override public void Start() {
		base.Start();
		this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
		this.complexity = 3;
	}
	
	// Update is called once per frame
	override public void Update () 
	{
		base.Update();
		
		var loc = this.transform.position;
		loc.y = 0;
		
		// Complexity 1 == dead in the water
		if (this.complexity <= 1)
		{
			if (!this.rigidbody.IsSleeping())
				this.rigidbody.AddForce(Vector3.zero);
			
			return;
		}
		
		
		if (this.complexity < 3 && this.target)
		{
			// Cannot follow other cubes at compl < 3
			this.target = null;
			this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
		}
		
		if (this.complexity >= 3 && this.target == null)
		{
			// Look out for things to follow
			var hits = Physics.SphereCastAll(this.transform.position, 300f, Vector3.up, 1, 1 << LayerMask.NameToLayer("BigCubes"));
			
			foreach (var h in hits)
			{
				var cube = h.collider.GetComponent<CubeBehavior>();
				if (cube != null && cube.complexity >= 3 && cube != this && cube.target == null)
				{
					this.target = cube.transform;
					break;
				}					
			}
		}
		if (this.target && this.target.GetComponent<CubeBehavior>().complexity < 3)
		{
			// Will not follow cubes with complexity < 3
			this.target = null;
			this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
		}
		
		
		var forceSize = this.target ? this.followForceSize : this.forceSize;		
		
		if (this.target)
		{
			this.targetLocation = this.target.position;
			this.targetLocation.y = 0;
			
			// Slow down if near target, not to overshoot
			if ((this.targetLocation - loc).magnitude < 10) {
				forceSize = forceSize * (this.targetLocation - loc).magnitude / 10;
			}
		}
		else if ((this.targetLocation - loc).magnitude < 60)
		{
			// Arrived ? Get new random location
			this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
		}

		var dir = (this.targetLocation - loc).normalized;
		//var dotDir = Vector3.Dot(dir, this.rigidbody.velocity.normalized);		
		
		if (this.rigidbody.velocity.magnitude < (this.target ? this.followMaxSpeed : this.maxSpeed))
		{
			// Only steer if not too fast or other direction...
		
			var topEdge = this.transform.position + dir*-this.size/2 + new Vector3(0, this.size, 0f);
			var force = dir * forceSize + new Vector3(0, this.rigidbody.velocity.magnitude < this.maxSpeed/2 ? this.spikeUpForce : this.upForce, 0);
			this.rigidbody.AddForceAtPosition(force, topEdge);
		}
		
		
		if (this.playerPower.power < -2)
		{
			// Aggro mode!
			var dif = (this.player.position - this.transform.position);
			
			//this.renderer.material.color = this.normalColor;
			var wasHit = false;
			/*
			if (dif.magnitude < 50f)
			{
				RaycastHit hit;
				if (Physics.Raycast(this.transform.position, dif, out hit, 50f, -1))
				{
					if (hit.collider.transform == this.player)
					{
						wasHit = true;
						this.renderer.material.color = Color.red + Color.yellow;
						
						if (!this.beam)
						{
							this.beam = (GameObject)Instantiate(Resources.Load("Beam"));
							this.beam.transform.parent = this.transform;
							this.beam.transform.localPosition = Vector3.zero;
						}
						LineRenderer l = this.beam.GetComponent<LineRenderer>();
						if (!this.beam.activeSelf)
						{
							this.beam.SetActive(true);
							this.beamOffset = new Vector3(Random.value - 0.5f, Random.value - 0.5f, Random.value - 0.5f).normalized * 0.5f;
						}
						//l.SetPosition(1, this.transform.InverseTransformPoint(hitInfo.point));
						var dist = hit.point - this.transform.position;
						l.SetPosition(0, this.transform.position);
						l.SetPosition(1, this.transform.position + (dist.normalized * (dist.magnitude + 1f)) + beamOffset);
						
						this.player.rigidbody.AddForceAtPosition(dif.normalized * 20f, hit.point);
					}
				}
			}
			
			if (!wasHit && this.beam && this.beam.activeSelf == true)
				this.beam.SetActive(false);
				*/
		}
	}
	
	override public bool Affect(int power, RaycastHit hit)
	{
		// Negs
		if (power < 0 && complexity == 3 && hit.distance < 50f)
		{
			this.complexity--;
		}
		else if (power < -2 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity--;
		}
		// Pos
		else if (power > 0 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity++;
		}
		else
		{
			return false;	
		}
		
		switch (this.complexity)
		{
			case 1:	
				this.GetComponent<Renderer>().material.color = Color.white; 
				this.transform.localScale = new Vector3(30, 30, 30);
				this.size = 30;
				break;
			case 2:	
				this.GetComponent<Renderer>().material.color = new Color(0.6f, 0.5f, 0.3f); ; 
				this.transform.localScale = new Vector3(40, 40, 40);
				this.size = 40;
				break;
			case 3: 
				this.GetComponent<Renderer>().material.color = new Color(0.5f, 0.25f, 0); 
				this.transform.localScale = new Vector3(60, 60, 60);
				this.size = 60;
				break;
		}
		
		return true;
	}
}
