using UnityEngine;
using System.Collections;

public class ConeBehavior : Affector {
	
	public Vector3 targetLocation;
	public Transform target;
	public float forceSize = 10f;
	public float maxSpeed = 20f;
	
	public float followForceSize = 20f;
	public float followMaxSpeed = 30f;
	
	public float preferredHeight = 85f;
	public float targetHeightMin = 100f;
	public float targetHeightMax = 120f;
	
	Transform nose;
	
	// Use this for initialization
	override public void Start() {
		base.Start();
		this.targetLocation = new Vector3(Random.value*2000f-1000, Random.Range(this.targetHeightMin, this.targetHeightMax), Random.value*2000f-1000);
		this.complexity = 3;
		this.nose = this.gameObject.GetComponentInChildren<Transform>();
	}
	
	// Update is called once per frame
	override public void Update ()
	{
		base.Update();
		
		// Complexity 1 == deadish
		if (this.complexity <= 1)
		{
			if (!this.rigidbody.IsSleeping())
				this.rigidbody.AddForce(Vector3.zero);
			
			return;
		}		
		
		var loc = this.transform.position;
		
		if (this.playerPower.power < -2)
		{
			// Aggro mode!
			var dif = (this.player.position - this.transform.position);
			
			var wasHit = false;
				
			if (dif.magnitude < 300f)
			{
				RaycastHit hit;
				if (Physics.Raycast(this.transform.position, dif, out hit, 300f, -1))
				{
					this.renderer.material.color = Color.cyan;
					
					if (hit.collider.transform == this.player)
					{
						wasHit = true;
						//this.renderer.material.color = Color.red + Color.yellow;
						
						this.target = this.player;
					}
				}
			}
		}
		else
		{
			if (this.complexity < 3 && this.target)
			{
				// Cannot follow others at compl < 3
				this.target = null;
				this.targetLocation = new Vector3(Random.value*2000f-1000, Random.Range(this.targetHeightMin, this.targetHeightMax), Random.value*2000f-1000);
			}
			
			if (this.complexity >= 3 && this.target == null)
			{
				// Look out for things to follow
				var hits = Physics.SphereCastAll(this.transform.position, 300f, Vector3.up, 1, 1 << LayerMask.NameToLayer("Cones"));
				
				foreach (var h in hits)
				{
					var cone = h.collider.GetComponent<ConeBehavior>();
					if (cone != null && cone.complexity >= 3 && cone != this && cone.target == null)
					{
						this.target = cone.transform;
						break;
					}					
				}
			}
			if (this.target && this.target.GetComponent<ConeBehavior>().complexity < 3)
			{
				// Will not follow others with complexity < 3
				this.target = null;
				this.targetLocation = new Vector3(Random.value*2000f-1000, Random.Range(this.targetHeightMin, this.targetHeightMax), Random.value*2000f-1000);
			}
		}
		
		var forceSize = this.target == this.player ? this.followForceSize * 4 : (this.target ? this.followForceSize : this.forceSize);		
		
		if (this.target)
		{
			this.targetLocation = this.target.position;
			
			// Slow down if near target, not to overshoot
			if ((this.targetLocation - loc).magnitude < 20) {
				forceSize = forceSize * (this.targetLocation - loc).magnitude / 20;
			}
		}
		else if ((this.targetLocation - loc).magnitude < 20)
		{
			// Arrived ? Get new random location
			this.targetLocation = new Vector3(Random.value*2000f-1000, Random.Range(this.targetHeightMin, this.targetHeightMax), Random.value*2000f-1000);
		}
		
		var dir = (this.targetLocation - loc).normalized;
		
		var dotDir = Vector3.Dot(dir, this.rigidbody.velocity.normalized);		
		
		if (dotDir < 0.8f || this.rigidbody.velocity.magnitude < (this.target == this.player ? this.followMaxSpeed * 4 : (this.target ? this.followMaxSpeed : this.maxSpeed)))
		{
			// Only steer if not too fast or other direction...
			var force = dir * forceSize;
			this.rigidbody.AddForceAtPosition(force, this.nose.position);
		}
		
		var rotation = Quaternion.LookRotation(this.targetLocation - this.transform.position, this.transform.up);
        this.transform.rotation = Quaternion.Slerp(this.transform.rotation, rotation, Time.deltaTime);		
		
	}
	
	override public bool Affect(int power, RaycastHit hit)
	{
		// Nega power
		if (power < 0 && complexity == 3 && hit.distance < 50f)
		{
			this.complexity--;
		}
		else if (power < -2 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity--;
		}
		// Positive
		else if (power > 0 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity++;
		}
		else
		{
			return false;	
		}
		
		switch (this.complexity)
		{
			case 1:	
				this.GetComponent<Renderer>().material.color = Color.white; 
				this.rigidbody.useGravity = true;
				this.rigidbody.constraints = 0;
				break;
			case 2:	
				this.GetComponent<Renderer>().material.color = Color.grey; 
				this.rigidbody.useGravity = false;
			
				this.rigidbody.constraints =  RigidbodyConstraints.FreezeRotationX | RigidbodyConstraints.FreezeRotationY |  RigidbodyConstraints.FreezeRotationZ;
				break;
			case 3: 
				this.GetComponent<Renderer>().material.color = Color.green; 
				break;
		}
		
		return true;		
	}
	
	public void OnCollisionEnter(Collision c)
	{
		if (c.gameObject.name == "Terrain")
		{
			// Crash? get new direction!
			this.targetLocation = new Vector3(Random.value*2000f-1000, Random.Range(this.targetHeightMin, this.targetHeightMax), Random.value*2000f-1000);
		}
		else if (c.transform == this.player && this.complexity > 1)
		{
			this.player.rigidbody.AddForceAtPosition(c.contacts[0].normal * -300f, c.contacts[0].point, ForceMode.Impulse);
		}
	}
}
