using UnityEngine;
using System.Collections;

public class SphereBehavior : Affector {
	
	public Vector3 targetLocation;
	public Transform target;
	public float forceSize = 15f;
	public float maxSpeed = 20f;
	
	public float followForceSize = 30f;
	public float followMaxSpeed = 30f;
	
	bool fleeing = false;
		
	Texture tmpTexture;
	
	Color naturalColor = Color.black;
	
	// Use this for initialization
	override public void Start() {
		base.Start();
		this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
		this.complexity = 3;
	}
	
	// Update is called once per frame
	override public void Update ()
	{
		base.Update();
		
		var loc = this.transform.position;
		loc.y = 0;
		
		// Complexity 1 == dead in the water
		if (this.complexity <= 1)
		{
			if (!this.rigidbody.IsSleeping())
				this.rigidbody.AddForce(Vector3.up);
			
			return;
		}
		
		
		if (this.playerPower.power < -2)
		{
			// Flee mode!
			var dif = (this.player.position - this.transform.position);
			
			var seePlayer = false;
			if (dif.magnitude < 100f)
			{
				RaycastHit hit;
				if (Physics.Raycast(this.transform.position, dif, out hit, 100f, -1))
				{
					this.fleeing = true;
					seePlayer = true;
					
					dif = dif.normalized;
					dif.y = 0;
					
					this.targetLocation = this.transform.position + -dif * 200f;
				}
			}
			
			if (this.fleeing && !seePlayer)
			{
				this.fleeing = false;
				this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
			}
				
		}
		
		var forceSize = this.fleeing ? this.followForceSize : this.forceSize;
		
		
		if (this.target)
		{
			this.targetLocation = this.target.position;
			this.targetLocation.y = 0;
			
			if ((this.targetLocation - loc).magnitude < 10) {
				forceSize = forceSize * (this.targetLocation - loc).magnitude / 10;
			}
		}
		else if ((this.targetLocation - loc).magnitude < 40)
		{
			// Arrived ? Get new random location
			this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
		}
		
		var dir = (this.targetLocation - loc).normalized;
		var dotDir = Vector3.Dot(dir, this.rigidbody.velocity.normalized);		
		
		if (dotDir < 0 || this.rigidbody.velocity.magnitude < (this.target || this.fleeing ? this.followMaxSpeed : this.maxSpeed))
		{
			// Only steer if not too fast or other direction...
			
			var topEdge = this.transform.position + dir*-0.5f + new Vector3(0, 1f, 0f);
			var force = dir * forceSize + new Vector3(0, 15f, 0);
			this.rigidbody.AddForceAtPosition(force, topEdge);
		}
		
		
	}
	
	override public bool Affect(int power, RaycastHit hit)
	{
		// Negs
		if (power < 0 && complexity == 3 && hit.distance < 50f)
		{
			this.complexity--;
		}
		else if (power < -2 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity--;
		}
		// Pos
		else if (power > 0 && complexity == 2 && hit.distance < 50f)
		{
			this.complexity++;
		}
		else
		{
			return false;	
		}
		
		switch (this.complexity)
		{
			case 1:	
				this.tmpTexture = this.GetComponent<Renderer>().material.mainTexture;
				this.naturalColor = this.renderer.material.color;
				this.rigidbody.mass = 0.1f;
				this.rigidbody.useGravity = false;
				this.renderer.material.color = new Color(1f, 1f, 1f, 0.5f);   
				break;
			case 2:	
				this.tmpTexture = this.GetComponent<Renderer>().material.mainTexture;
				this.GetComponent<Renderer>().material.mainTexture = null;   
				
				if (this.naturalColor != Color.black)
					this.renderer.material.color = this.naturalColor;
			
				this.rigidbody.mass = 8;
				this.maxSpeed = 10;
				this.rigidbody.useGravity = true;
				this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
				break;
			case 3: 
				this.GetComponent<Renderer>().material.mainTexture = this.tmpTexture; 
				this.maxSpeed = 20;
				this.rigidbody.mass = 0.5f;
				this.targetLocation = new Vector3(Random.value*2000f-1000, 0, Random.value*2000f-1000);
				break;
		}
		
		return true;
		
	}
}
